# -*- coding: utf-8 -*-
# figur.py
# wegen der leichteren Lesbarkeit keine Kapselung ???

from grafikfenster import *
from math import radians

from abc import * # abstrakte Klasse realisieren

import platform
versionsNummer=platform.python_version()[0]

##### ===== Beginn Klassenkopf fuer Python 2 ===================
##if versionsNummer=='3':
##    print("Bitte diesen Bereich fuer Python 2 auskommentieren")
##    print("und den Bereich fuer Python 3 einkommentieren.")
##class Figur():
##    """Abstrakte Oberklasse fuer alle Figuren in Py 2.7
##    @author Claus Albowski
##    @version November 18"""
##    __metaclass__ = ABCMeta
##### ===== Ende Klassenkopf fuer Python 2 =====================

### ===== Beginn Klassenkopf fuer Python 3 ===================
if versionsNummer=='2':
    print("Bitte diesen Bereich fuer Python 3 auskommentieren")
    print("und den Bereich fuer Python 2 einkommentieren.")
class Figur(metaclass = ABCMeta):
    """Abstrakte Oberklasse fuer alle Figuren in Py 2.7
    @author Claus Albowski
    @version November 18"""
### ===== Ende Klassenkopf fuer Python 3 =====================

    def __init__(self, x=0, y=0,
                 breite=1, hoehe=1,
                 winkel=0, farbe='black', fuellfarbe=None):
        """Konstruktor"""
        self.__x, self.__y, self.__winkel = x,y,winkel
        self.__breite, self.__hoehe = breite,hoehe
        self.__farbe = farbe
        self.__fuellfarbe = fuellfarbe
        self.__sichtbar = True
        self.__gruppe = None
        self.Zeige()
        
    @abstractmethod
    def GibFigur(self):
        """Definiert den Pfad."""
        return NotImplemented

    ## Erweiterungen wegen der Textobjekte:
    def IstText(self):
        '''Zur eindeutigen Identifikation von Textobjekten'''
        return False

    def Transformiere(self, path):
        """Transformiert den übergebenen Pfad"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        gc.PushState()
        # Hilfvariable verwenden
        x,y = self.GibXPosition(), self.GibYPosition()
        w = self.GibOrientierung()
        b,h = self.GibBreite(), self.GibHoehe()
        gc.Translate(x+b/2, y+h/2)
        gc.Rotate(radians(w))
        gc.Translate(-b/2, -h/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        if self.__gruppe != None:
            return self.__gruppe.Transformiere(path)
        return path

    def SetzeGruppe(self, gruppe):
        self.__gruppe = gruppe

    def GibGruppe(self):
        return self.__gruppe

    def GibXPosition(self):
        """Get-Methode fuer die xPosition"""
        return self.__x
    
    def GibYPosition(self):
        """Get-Methode fuer die yPosition"""
        return self.__y
    
    def GibBreite(self):
        """Get-Methode fuer die Breite (Klassenvariable)"""
        return self.__breite
    
    def GibHoehe(self):
        """Get-Methode fuer die Hoehe"""
        return self.__hoehe
    
    def GibOrientierung(self):
        """Get-Methode fuer die Orientierung"""
        return self.__winkel
    
    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        return self.__farbe
    
    def GibFuellFarbe(self):
        """Get-Methode fuer die Fuellfarbe
        gibt None zurueck, wenn nur der Umriss gezeichnet
        werden soll"""
        return self.__fuellfarbe
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.__sichtbar

    def Bewege(self, umX=0, umY=0):
        """Veraendernde Methode fuer die x- und y-Position"""
        self.Verberge()
        self.__x += umX
        self.__y += umY
        self.Zeige()
    
    def AendereAbmessungen(self, neueBreite, neueHoehe):
        """Veraendernde Methode fuer die Breite und Hoehe"""
        self.Verberge()
        self.__breite = neueBreite
        self.__hoehe = neueHoehe
        self.Zeige()
    
    def Skaliere(self, skalierungsFaktor):
        """Veraendernde Methode Breite und Hoehe mit gleichem Faktor"""
        self.Verberge()
        self.__breite *= skalierungsFaktor
        self.__hoehe *= skalierungsFaktor
        self.Zeige()
    
    def Drehe(self, umWinkel=0):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Verberge()
        self.__winkel += umWinkel
        self.Zeige()

    def AendereFarbe(self, neueFarbe='black'):
        """Veraendernde Methode fuer die Farbe
        Standard ist black"""
        self.Verberge()
        self.__farbe = neueFarbe
        self.Zeige()
    
    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Veraendernde Methode fuer die Fuellfarbe
        ohne Parameter wird die Fuellung geloescht."""
        self.Verberge()
        self.__fuellfarbe = neueFuellfarbe
        self.Zeige()
    
    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__sichtbar = False
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__sichtbar = True
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class FigurApp(wx.App):
    """Test-Anwendung"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Figuren-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung:
        sollte nicht gehen!"""
        global figur
        try:
            figur=Figur(30, 30, 40, 40, 270, 'blue')
        except Exception as e:
            print( str(e) )
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = FigurApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
