# figurenGruppe.py

from grafikfenster import *

from figur import Figur

### ---------------------------------------------------
class FigurenGruppe(Figur):
    """Klasse FigurenGruppe ermoeglicht das Zeichnen
    und Bearbeiten von FigurenGruppen.
    Die Kindobjekte werden mit relativer Position eingebaut."""

    def __init__(self,
                 xPos=0,
                 yPos=0,
                 winkel=0):
        self.__figuren = []
        Figur.__init__(self, xPos, yPos, 1, 1, winkel, 'black')
        
    def AendereAbmessungen(self, neueBreite, neueHoehe):
        """Veraendernde Methode fuer die Breite und Hoehe
        Die relativen Positionen werden korrigiert."""
        faktorX,faktorY = 1.0,1.0
        if self.GibBreite()>0:
            faktorX = float(neueBreite)/self.GibBreite()
        if self.GibHoehe()>0:
            faktorY = float(neueHoehe)/self.GibHoehe()
        self._Figur__breite = neueBreite
        self._Figur__hoehe = neueHoehe
        for figur in self.__figuren:
            b = faktorX * figur.GibBreite()
            h = faktorY * figur.GibHoehe()
            figur.AendereAbmessungen(b, h)
            dx = figur.GibXPosition() * (faktorX - 1.0)
            dy = figur.GibYPosition() * (faktorY - 1.0)
            figur.Bewege(dx, dy)
        
    def AendereFarbe(self, neueFarbe='black'):
        """Gruppen sollten ggf. keine eigene Farbe haben"""
        for figur in self.__figuren:
            figur.AendereFarbe(neueFarbe)
        
    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Gruppen sollten ggf. keine eigene Fuellfarbe haben"""
        print('Guppen sollten ggf. keine eigene Fuellfarbe haben')
        for figur in self.__figuren:
            figur.AendereFuellfarbe(neueFuellfarbe)
        
    def GibFigur(self):
        """Definiert den Pfad."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        return path

    def __AktualisiereAbmessungen(self):
        """Interne Methode zum Aktualisieren der Abmessungen
        nach Einbau oder Entfernen einer neuen Figur"""
        b,h = 1,1
        for figur in self.__figuren:
            b=max(figur.GibXPosition()+figur.GibBreite(), b)
            h=max(figur.GibYPosition()+figur.GibHoehe(), h)
        self._Figur__breite,self._Figur__hoehe=b,h

    def FuegeHinzu(self, figur):
        """fuegt der Gruppe eine Figur hinzu
        Voraussetzung fuer richtige Werte ist winkel=0 bei Kindern"""
        self.Verberge()
        if figur not in self.__figuren:
            self.__figuren.append(figur)
            figur.SetzeGruppe(self)
            self.__AktualisiereAbmessungen()
        self.Zeige()
    
    def Entferne(self, figur):
        """entfernt eine Figur aus der Gruppe"""
        self.Verberge()
        if figur in self.__figuren:
            self.__figuren.remove(figur)
            figur.SetzeGruppe(None)
            self.__AktualisiereAbmessungen()
        self.Zeige()
    
    def AendereReihenfolge(self, von, nach):
        """aendert die Reihenfolge, in der die Figuren
        der Gruppe dargestellt werden"""
        anz=len(self.__figuren)
        if (von<1) or (von>anz) or (nach<1) or (nach>anz) or (von==nach): return
        self.Verberge()
        tausch=self.__figuren[von-1]
        self.__figuren[von-1]=self.__figuren[nach-1]
        self.__figuren[nach-1]=tausch
        self.Zeige()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__sichtbar = False
        for figur in self.__figuren:
            figur.Verberge()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__sichtbar = True
        for figur in self.__figuren:
            figur.Zeige()

### ---------------------------------------------------
class GruppeApp(wx.App):
    """Test-Anwendung"""
    def OnInit(self):
        self.__fenster = GrafikFenster(None, "Figuren-Grafik")
        self.SetTopWindow(self.__fenster)
        self.__fenster.Show(True)
        self.__fenster.panel.Refresh()
        self.__fenster.ZeigeShellFrame()
##        self.TestAnwendung()
##        self.TestAnwendung2()
        self.TestAnwendung3()
##        self.TestAnwendung4()
##        self.TestAnwendung5()
        #self.__fenster.ZeigeFillingFrame()
        return True

    def BildExport(self, dateiname='Diagramm.png'):
        Zeichenflaeche.GibZeichenflaeche().BildExport(dateiname)

    def TestAnwendung(self):
        """Test-Methode speziell fuer Gruppen"""
        global gruppe, e1, e2, r1, r2
        from ellipse import Ellipse
        from rechteck import Rechteck
        gruppe=FigurenGruppe(120,170)
        r1=Rechteck(0, 0, 100, 60, 0, 'BLACK')
        gruppe.FuegeHinzu(r1)
        e1=Ellipse(0, 0, 100, 60, 0, 'RED')
        gruppe.FuegeHinzu(e1)
        e2=Ellipse(12.5, 7.5, 75, 45, 0, 'BLUE','YELLOW')
        gruppe.FuegeHinzu(e2)
        r2=Rechteck(20, 20, 100, 60, 0, 'MAGENTA')
        gruppe.FuegeHinzu(r2)

    def TestAnwendung2(self):
        """Test-Methode speziell fuer einen Pfeil"""
        global gruppe2, pfeil, kreis
        from ellipse import Ellipse
        from nEck import N_Eck
        gruppe2=FigurenGruppe(10,40)
        kreis=Ellipse(0, 0, 30, 30, 0, 'red')
        kreis.AendereFarbe('black')
        kreis.AendereFuellfarbe('light blue')
        gruppe2.FuegeHinzu(kreis)
        pfeil=N_Eck(7,5,((0,0),(20,10),(0,20),(10,10)))
        pfeil.AendereFarbe('black')
        pfeil.AendereFuellfarbe('blue')
        gruppe2.FuegeHinzu(pfeil)
        gruppe2.Skaliere(5)

    def TestAnwendung3(self):
        """Test-Methode speziell fuer einen Pfeil"""
        global gruppe3, pfeil, kreis
        from ellipse import Ellipse
        from nEck import N_Eck
        gruppe3=FigurenGruppe(60,40)
        kreis=Ellipse(0, 0, 30, 30, 0, 'red')
        kreis.AendereFarbe('black')
        kreis.AendereFuellfarbe('light blue')
        gruppe3.FuegeHinzu(kreis)
        pfeil=N_Eck(7,5,((0,0),(15,7),(15,0),(17,0),(17,20),(15,20),(15,13),(0,20)))
        pfeil.AendereFarbe('black')
        pfeil.AendereFuellfarbe('blue')
        gruppe3.FuegeHinzu(pfeil)
        gruppe3.Skaliere(5)

    def TestAnwendung4(self):
        """Test-Methode speziell fuer einen Pfeil"""
        global gruppe4, pfeil, kreis
        from ellipse import Ellipse
        from nEck import N_Eck
        gruppe4=FigurenGruppe(10,80)
        kreis=Ellipse(0, 0, 30, 30, 0, 'black')
        kreis.AendereFarbe('black')
        kreis.AendereFuellfarbe('light gray')
        gruppe4.FuegeHinzu(kreis)
        pfeil=N_Eck(7,5,((0,0),(20,10),(0,20),(10,10)))
        pfeil.AendereFarbe('black')
        pfeil.AendereFuellfarbe('gray')
        gruppe4.FuegeHinzu(pfeil)
        gruppe4.Skaliere(5)

    def TestAnwendung5(self):
        """Test-Methode speziell fuer einen Pfeil"""
        global gruppe5, pfeil, kreis
        from ellipse import Ellipse
        from nEck import N_Eck
        gruppe5=FigurenGruppe(60,80)
        kreis=Ellipse(0, 0, 30, 30, 0, 'black')
        kreis.AendereFuellfarbe('light gray')
        gruppe5.FuegeHinzu(kreis)
        pfeil=N_Eck(7,5,((0,0),(15,7),(15,0),(17,0),(17,20),(15,20),(15,13),(0,20)))
        pfeil.AendereFarbe('black')
        pfeil.AendereFuellfarbe('gray')
        gruppe5.FuegeHinzu(pfeil)
        gruppe5.Skaliere(5)

### ---------------------------------------------------
if __name__ == '__main__':
    app = GruppeApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
